from aleat3 import Aleatoryous, coinToBool, coinToBinary
from aleat3.output.colored import UNABLE

__all__ = ["main",
           "coin",
           "dice",
           "roulette"]

# Main demostration of Aleatoryous:
def main():
    # Based from Tests/aleatoryous.py
    a = Aleatoryous("aleatory.coin")
    print("coin " + str(a.single() in ["Head", "Tails"]))
    print("cointobinary " + str(coinToBinary(a.single()) in [0, 1]))
    print("cointobool " + str(coinToBool(a.single()) in [True, False]))
    a.changemode("aleatory.dice")
    print("dice " + str(a.single() in [0, 1, 2, 3, 4, 5, 6]))
    b = ["a", "b", "c"]
    a.changemode("aleatory.roulette", b)
    print("roulette " + str(a.single() in b))
    print("colorama output " + str(UNABLE))
    print("Done")

# coin():
def coin():
    c = Aleatoryous("aleatory.coin")
    cc = c.single()
    print("""coin single() result: %s
coinToBool function: %s
coinToBinary function: %s"""%(cc, coinToBool(cc), coinToBinary(cc)))

# dice():
def dice():
    d = Aleatoryous("aleatory.dice")
    print("dice single() result: %s"%d.single())

# roulette():
def roulette():
    l = ["Option 1", "Option 2", "Option 3"]
    print("Available options:")
    for ll in range(l):
        print(" - %s"%l[ll])
    r = Aleatoryous("aleatory.roulette", l)
    print("roulette single(): %s"%r.single())



# The whole syntax of Aleatoryous3

__all__ = ["InitError",
           "Aleatoryous",
           "coinToBinary",
           "coinToBool"]


import random

if __name__ != "__main__":
    from aleat3.output import init_errors as IE, colored as _color
else:
    from output import init_errors as IE, colored as _color


class InitError(TypeError):
    pass


"New since 0.1.5: Cleaner operations"
class Aleatoryous:
    "New since 0.0.4: Some variables deleted or recycled"
    "New since 0.0.9: More variables recycled"
    tot = 0
    #__cache = ["Coin", "Dice", "Roulette"]
    __mode = ""
    parser = ""
    lst = []
    __methods = ["__init__",
                 "__len__",
                 "__dir__",
                 "refresh",
                 "single",
                 "first_5",
                 "first_10",
                 "first_50",
                 "first_100",
                 "first_given",
                 "getmode",
                 "changemode"]
    dict = {}
    __it = 0

    def __init__(self, mode="aleatory.coin", extras=None):
        if mode == "aleatory.coin":
            self.__mode = "Coin"
            self.__it = 2
        elif mode == "aleatory.dice":
            self.__mode = "Dice"
            self.__it = 6
        elif mode == "aleatory.roulette":
            self.__mode = "Roulette"
            if extras is not None:
                self.parser = extras
            if not isinstance(self.parser, type([1, 2, 3])):
                raise InitError(IE.parameter_bug())
            else:
                pass
            for i in range(len(self.parser)):
                self.it = i + 1
                self.dict[self.parser[i]] = self.it
            self.__it = len(self.parser)
        else:
            raise InitError(IE.modal_bug())
        #self.cache = ""
        self.__modetye = mode
        self.parser = ""


    def refresh(self):
        self.tot = ""
        self.lst = []


    def single(self):
        if self.__mode == "Coin":
            self.tot = random.randint(0, 1)
            if self.tot == 1:
                return "Head"
            elif self.tot == 0:
                return "Tails"
            else:
                return None

        elif self.__mode == "Dice":
            self.tot = random.randint(1, 6)
            return self.tot

        elif self.__mode == "Roulette":
            self.tot = random.randint(1, len(self.dict))
            for k, v in self.dict.items():
                if v == self.tot:
                    self.tot = k
                    break
            return self.tot


    def first_5_basic(self):
        "New since 0.0.6: Faster operations"
        for time in [1, 2, 3, 4, 5]:
            self.lst.append(self.single())


    def first_5(self):
        self.refresh()
        self.first_5_basic()
        return self.lst


    def first_10(self):
        self.refresh()
        for time in [1, 2]:
            self.first_5_basic()
        return self.lst


    def first_50(self):
        self.refresh()
        for chance in [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]:
            self.first_5_basic()
        return self.lst


    def first_100(self):
        self.refresh()
        for time in [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20]:
            self.first_5_basic()
        return self.lst


    def first_given(self, how, /, repeat=True):
        "New since 0.0.6: You can iterate with no repetition"
        "New since 0.0.7: No-repetition operation corrected"
        "New since 0.1.3: No-repetition operation corrected"
        self.refresh()
        if not isinstance(how, type(1)):
            raise SyntaxError(f"Expected integers, %s given" % type(how))
        else:
            if self.__mode == "Coin" and repeat is False:
                raise SyntaxError(f"'No-repetition' feature is not available for 'aleatory.dice' objects")
            if not repeat and how > self.__it:
                raise SyntaxError(f"Cannot give more than %s iterations without repetitions." % self.__it)
        if repeat is True:
            for time in range(how):
                self.lst.append(self.single())
        elif repeat is False:
            while len(self.lst) != how:
                possible = self.single()
                if possible in self.lst:
                    continue
                self.lst.append(possible)
        else:
            raise SyntaxError(f"At parameter 'repeat', you must enter True or False, not %s" % repeat)
        return self.lst


    def getmode(self):
        "New since 0.0.3: get the private mode."
        self.refresh()
        return self.__modetype


    def changemode(self, mode, extras=None):
        "New since 0.0.3: restart the object with a new mode."
        self.__init__(mode, extras)

    # Some built-in functions
    def __len__(self):
        return self.__it  # The number of options
    def __dir__(self):
        return self.__methods
    def __str__(self):
        return "Aleatoryous " + self.__modetype


#################################################################################################################################################
def coinToBinary(res):
    if res.strip().lower() == "head":
        return 1
    elif res.strip().lower() == "tails":
        return 0
    else:
        _color.output_red("ERROR: You must enter an aleatory.dice string output for binary conversion. The default value will be set as None.")
        return None

def coinToBool(res):
    "New since 0.0.8: From string to boolean"
    e = coinToBinary(res)
    if e == 1:
        return True
    elif e == 0:
        return False
    else:
        _color.output_red("ERROR: You must enter an aleatory.dice string output for boolean conversion. The default value will be set as None.")


#################################################################################################################################################
"New since 0.1.1: Module test"

def module_test():
    import time
    _color.output_magenta("----Module test: constructor.py----")
    #print("----Module test----\n")
    print("Testing module...\n")

    try:
        print("Testing objects...")
        time.sleep(0.7)
        try:
            print("-Testing object 'aleat3.Aleatoryous'...")
            i = Aleatoryous()
            i.changemode("aleatory.roulette", [1, 2, 3])
            i.changemode("aleatory.dice")
            i.changemode("aleatory.coin")
            _color.output_green(" The object 'aleat3.Aleatoryous' is OK.")
        except:
            _color.output_red("ERROR: Something's wrong at object 'aleat3.Aleatoryous' test.")

        try:
            print("-Testing object 'aleat3.InitError'...")
            class killme:
                def __init__(self):
                    raise InitError(f"Test info")
            _color.output_green(" The object 'aleat3.InitError' is OK.")
        except:
            _color.output_red("ERROR: Something's wrong at object 'aleat3.InitError' test.")

        print("Testing coin conversions...")
        time.sleep(0.7)
        print("-Testing 'coinToBinary'...")
        if coinToBinary(i.single()) in [1, 0]:
            _color.output_green(" The coin conversion 'coinToBinary' is OK.")
        else:
            _color.output_red("ERROR: Something's wrong at coin conversion 'coinToBinary'.")

        print("-Testing 'coinToBool'...")
        if coinToBool(i.single()) in [True, False]:
            _color.output_green(" The coin conversion 'coinToBool' is OK.")
        else:
            _color.output_red("ERROR: Something's wrong at coin conversion 'coinToBool'.")
        _color.output_green("The module is OK.")
    except:
        _color.output_red("ERROR: Something's wrong in the module.")

    _color.output_magenta("----Test finshed----")
    #print("----Test finished----\n")
    time.sleep(1)
    i = input("Done")


# ifmain function
if __name__ == '__main__':
    _color.output_yellow("NOTE: When using this file as __main__ level, you are executing the module test. This operation may take some minutes.")
    import time
    time.sleep(1.5)
    module_test()



"An example of using aleatory.dice in Tkinter interfaces."

if __name__ == '__main__':
    raise Exception(f"Unable to load application file as __main__ level")

from aleat3 import Aleatoryous
from tkinter import *

__all__ = ["run"]
__author__ = "Diego Ramirez (dr01191115@gmail.com) @DiddiLeija"

class diceInterface:
    "A simple interface that uses aleatory.dice mode and Tkinter library."

    def __init__(self, root, title="aleatory.dice: Tkinter example"):
        self.root = root
        self.root.title(title)
        self.root.maxsize(500, 110)
        self.dice_s = Aleatoryous("aleatory.dice")
        self.principal()

    def principal(self):
        self.frame = Frame(self.root)
        self.frame.grid()
        self.dice_i = Label(self.frame,
                            text="6",
                            font=("Times", "50", "bold"),
                            fg="gray",
                            bg="white",
                            bd=1,
                            width=5)
        self.dice_i.grid(row=0, column=0)
        self.start = Button(self.frame,
                            text="Roll Dice",
                            fg="green",
                            bg="white",
                            command=self.roll_dice,
                            anchor="s")
        self.start.grid(row=1, column=0)
        self.info = Label(self.frame,
                          text="""Use the aleatory.dice syntax
for creating dice games easier.
If you want, use a dice image instead of tkinter.Label()""",
                          fg="black")
        self.info.grid(row=0, column=1)
        self.exit = Button(self.frame,
                           text="Close example",
                           fg="red",
                           bg="white",
                           command=self.root.quit,
                           anchor="s")
        self.exit.grid(row=1, column=1)

    def roll_dice(self):
        tot = str(self.dice_s.single())
        self.dice_i.config(text=tot, fg="blue")


def run(title="aleatory.dice: Tkinter example"):
    _r = Tk()
    _inst = diceInterface(_r, title)
    _r.mainloop()
    i = input("Done")



"""At this place, we give only the messages for raising."""

def parameter_bug():
    return f"__init__() Invalid Syntax (Unexpected parameter given: extras)"

def KI_bug():
    return f"Program Interrupted by User..."

def modal_bug():
    return f"__init__() Invalid Syntax (Unexpected parameter given: mode)"

######################################################################################################################################################
def module_test(main_level=False):
    tmp = [f"__init__() Invalid Syntax (Unexpected parameter given: mode)",
           f"Program Interrupted by User...",
           f"__init__() Invalid Syntax (Unexpected parameter given: extras)"]
    if main_level:
        from colored import output_red, output_green, output_magenta
    else:
        from aleat3.output.colored import output_red, output_green, output_magenta
    output_magenta("----Module test: init_errors.py----")
    print("Testing functions...\n")
    if parameter_bug() == tmp[2]:
        output_green("-parameter_bug OK")
        if KI_bug() == tmp[1]:
            output_green("-ki_bug OK")
            if modal_bug() == tmp[0]:
                output_green("-modal_bug OK")
                output_green("The module is OK.")
    else:
        output_red("ERROR: Something failed at test.")
    output_magenta("----Test finished----")
    i = input("Done")

if __name__ == '__main__':
    from colored import output_yellow
    output_yellow("NOTE: When using this file as __main__ level, you are executing the module test. This operation may take some minutes.")
    module_test(True)



import setuptools

#from setuptools.config import read_configuration
#conf_dict = read_configuration("C:/Program Files/Ramz Editions/Packaaging Examples/Aleatoryous 3/0.2/setup.cfg")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aleat3",
    version="0.2a3",
    author="Diego Ramirez",
    author_email="dr01191115@gmail.com",
    description="An aleatory syntaxes package. Third generation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Customer Service",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Games/Entertainment",
        "Topic :: Software Development"
    ],
    keywords="aleatory dice coin roulette python aleat",
    python_requires='>=3.6, <4',
    entry_points={
        "console_scripts": [
            "aleat3=aleat3.main:main",
            "aleat3_coin=aleat3.main:coin",
            "aleat3_dice=aleat3.main:dice",
            "aleat3_roulette=aleat3.main:roulette"
        ]
    },
    install_requires=["wheel>=0.36.0",
                      "setuptools>=50.3.1",
                      "pip>=21.0"]
)
